#define UNICODE
#define _UNICODE

#include <windows.h>
#include <winperf.h>   // for Windows NT perf mon support 

#define REGKEY_WINLOGON  \
   TEXT("software\\microsoft\\windows nt\\currentversion\\winlogon")
#define REGKEY_PERF  \
   TEXT("software\\microsoft\\windows nt\\currentversion\\perflib") 

HANDLE  hDoneEvent = NULL, hThread = NULL, hThread2 = NULL;
DWORD   dwCurrentState;
SERVICE_STATUS_HANDLE  hService;
TCHAR szCaption[] = TEXT("WDJSrvc");

BOOL NotifySCM(DWORD, DWORD, DWORD);
VOID WINAPI ServiceMain(DWORD, LPTSTR *);
VOID WINAPI ServiceHandler(DWORD);
DWORD WINAPI WorkerThread(LPVOID);
DWORD WINAPI PerfMonThread(LPVOID);
BOOL IsTaskRunning(LPTSTR, LPTSTR);
BOOL GetCounterNumber(LPTSTR, LPTSTR);

//-----------------------------------------------------------------
void main(int argc, char **argv)
{
   SERVICE_TABLE_ENTRY ServiceTable[] = {
      {TEXT("WDJSrvc"), ServiceMain}, {NULL, NULL}
   };  
   StartServiceCtrlDispatcher(ServiceTable);
}

//-----------------------------------------------------------------
VOID WINAPI ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
{
   DWORD   ThreadId;

   if (!(hService = RegisterServiceCtrlHandler(
         TEXT("WDJSrvc"), ServiceHandler))) {
      return;
   }

   NotifySCM(SERVICE_START_PENDING, 0, 1);

   hDoneEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
   hThread = CreateThread(0,0,WorkerThread,0,0,&ThreadId);
   hThread2 = CreateThread(0,0,PerfMonThread,0,0,&ThreadId);
   
   NotifySCM(SERVICE_RUNNING, 0, 0);
   WaitForSingleObject(hDoneEvent, INFINITE);

   CloseHandle(hThread);
   CloseHandle(hDoneEvent);
   return;
}

//--------------------------------------------------------------
VOID WINAPI ServiceHandler(DWORD fdwControl)
{
   switch(fdwControl) {
      case SERVICE_CONTROL_STOP:
         NotifySCM(SERVICE_STOP_PENDING, 0, 1);
         SetEvent(hDoneEvent);
         NotifySCM(SERVICE_STOPPED, 0, 0);
         break;

      case SERVICE_CONTROL_INTERROGATE:
         NotifySCM(dwCurrentState, 0, 0);
         break;

      case SERVICE_CONTROL_SHUTDOWN:
         break;
   }
}

//--------------------------------------------------------------
BOOL NotifySCM(DWORD State, DWORD ExitCode, DWORD Progress)
{
   SERVICE_STATUS ServiceStatus;

   ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
   ServiceStatus.dwCurrentState = dwCurrentState = State;
   ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP |
                                      SERVICE_ACCEPT_SHUTDOWN;
   ServiceStatus.dwWin32ExitCode = ExitCode;
   ServiceStatus.dwServiceSpecificExitCode = 0;
   ServiceStatus.dwCheckPoint = Progress;
   ServiceStatus.dwWaitHint = 5000;

   return SetServiceStatus(hService, &ServiceStatus);
}

//-----------------------------------------------------------------
DWORD WINAPI WorkerThread(LPDWORD ThreadParam)
{
   while (TRUE) {                                   
      Sleep(5000);
   }
   return 0;
}

//-----------------------------------------------------------------
DWORD WINAPI PerfMonThread(LPDWORD ThreadParam)
{   
    BOOL  bFound, bLastState = FALSE;
    DWORD dwType, dwSize, i;
    HKEY  hKey;
    TCHAR szShell[MAX_PATH], szCounterNumber[MAX_PATH];
    
    // Get winlogon process name.

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REGKEY_WINLOGON, 0, 
                     KEY_READ, &hKey) != ERROR_SUCCESS) {
        return 0;
    }

    dwSize = MAX_PATH;
    if (RegQueryValueEx(hKey, TEXT("shell"), NULL, &dwType, 
                        (LPBYTE)szShell,&dwSize) != ERROR_SUCCESS) {
        RegCloseKey(hKey);
        return 0;
    }

    // This is a comma delimited string, pick off first app.

    for (i=0; szShell[i] != 0; i++) {
        if (szShell[i] == ',') {
            szShell[i] = 0;
            break;
        }
    }
    RegCloseKey(hKey);

    // Retrieve the counter number associated with "Process" counter

    GetCounterNumber(TEXT("process"), szCounterNumber);

    // Poll task list, checking if shell task is running

    while (TRUE) {
        bFound = IsTaskRunning(szShell, szCounterNumber);
        if (bFound) MessageBeep(0);
        if (bFound != bLastState) {
            if (bFound) {
                MessageBox(NULL,TEXT("User logged on"),szCaption,
                           MB_OK|MB_SERVICE_NOTIFICATION);
            } else {
                MessageBox(NULL,TEXT("User logged off"),szCaption,
                           MB_OK|MB_SERVICE_NOTIFICATION);
            }
            bLastState = bFound;
        }
        Sleep(5000);
    }
    return 0;
}

//-----------------------------------------------------------------
BOOL GetCounterNumber(LPTSTR pszCounterName,LPTSTR pszCounterNumber)
{
    TCHAR   szTemp[MAX_PATH];
    LPBYTE  buf = NULL; 
    HKEY    hKey = NULL;
    DWORD   dwType, dwSize;
    LPTSTR  p, p2; 

    // Get list of counters using neutral English version.

    wsprintf(szTemp, TEXT("%s\\%03x"), REGKEY_PERF, 
             MAKELANGID(LANG_ENGLISH, SUBLANG_NEUTRAL)); 
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szTemp, 0, 
                     KEY_READ, &hKey) != ERROR_SUCCESS) { 
        goto exit; 
    }
 
    // Get buffer size for the counter names and allocate buffer 

    if (RegQueryValueEx(hKey, TEXT("Counters"), NULL, &dwType, 
                        NULL, &dwSize) != ERROR_SUCCESS) { 
        goto exit; 
    } 
 
    buf = (LPBYTE) malloc(dwSize); 
    if (buf == NULL) { 
        goto exit; 
    } 
    memset(buf, 0, dwSize); 
 
    // read the counter names from the registry 

    if (RegQueryValueEx(hKey, TEXT("Counters"), NULL, &dwType, 
                        buf, &dwSize) != ERROR_SUCCESS) { 
        goto exit; 
    } 
 
    // search multi_sz counter number, counter name pairs for 
    // "process" counter name
 
    p = (LPTSTR)buf; 
    while (*p) { 
        if (p > (LPTSTR)buf) { 
            for (p2=p-2; isdigit(*p2); p2--) ; 
        } 
        if (lstrcmpi(p, pszCounterName) == 0) { 
            // look backwards for the counter number 
            for (p2=p-2; isdigit(*p2); p2--) ; 
            lstrcpy(pszCounterNumber, p2+1); 
            break;
        } 
        p += (lstrlen(p) + 1); 
    } 
 
exit:
    if (buf) free(buf);
    if (hKey) RegCloseKey(hKey); 
    return TRUE;

} // GetCounterNumber

//-----------------------------------------------------------------
BOOL IsTaskRunning(LPTSTR pszShell, LPTSTR pszCounterNumber) 
{ 
    DWORD   rc, dwType, dwSize, dwNumTasks, i;
    TCHAR   szProcessName[MAX_PATH]; 
    LPBYTE  buf = NULL; 
    LPTSTR  p = NULL; 
    BOOL    bFound = FALSE;
    PPERF_DATA_BLOCK             pPerf; 
    PPERF_OBJECT_TYPE            pObj; 
    PPERF_INSTANCE_DEFINITION    pInst; 
    PPERF_COUNTER_BLOCK          pCounter; 

    // allocate the initial buffer for the performance data 

    dwSize = 49152;   // 48KB
    buf = malloc(dwSize); 
    if (buf == NULL) { 
        goto exit; 
    } 
    memset(buf, 0, dwSize); 
 
    while (TRUE) { 
        rc = RegQueryValueEx(HKEY_PERFORMANCE_DATA, pszCounterNumber,
                             NULL, &dwType, buf, &dwSize); 
 
        pPerf = (PPERF_DATA_BLOCK) buf; 
 
        // check for success and valid perf data block signature 
        if ((rc == ERROR_SUCCESS) && (dwSize > 0) && 
            (pPerf)->Signature[0] == (WCHAR)'P' && 
            (pPerf)->Signature[1] == (WCHAR)'E' && 
            (pPerf)->Signature[2] == (WCHAR)'R' && 
            (pPerf)->Signature[3] == (WCHAR)'F' ) { 
            break; 
        } 

        // if buffer is not big enough, reallocate and try again 

        if (rc == ERROR_MORE_DATA) { 
            dwSize += 24576;   // 24KB 
            buf = realloc(buf, dwSize); 
            memset(buf, 0, dwSize); 
        } else { 
            goto exit; 
        } 
    } 

    // search performance instance data, checking each process name 
 
    pObj = (PPERF_OBJECT_TYPE) ((DWORD)pPerf + pPerf->HeaderLength); 
    pInst = (PPERF_INSTANCE_DEFINITION)
                            ((DWORD)pObj + pObj->DefinitionLength); 
    dwNumTasks = (DWORD)pObj->NumInstances; 

    for (i=0; i<dwNumTasks; i++) { 
        // pointer to the process name 
        p = (LPTSTR) ((DWORD)pInst + pInst->NameOffset); 

        if (lstrlen(p)+4 <= sizeof(szProcessName)) { 
            lstrcpy(szProcessName, p); 
            lstrcat(szProcessName, TEXT(".exe")); 
        } 
        
        if (lstrcmpi(pszShell, szProcessName) == 0) {
            bFound = TRUE;
            goto exit;
        }
        
        pCounter = (PPERF_COUNTER_BLOCK) 
                         ((DWORD)pInst + pInst->ByteLength); 
        pInst = (PPERF_INSTANCE_DEFINITION) 
                         ((DWORD)pCounter + pCounter->ByteLength); 
    } 
 
exit: 
    if (buf) free(buf);  
    RegCloseKey(HKEY_PERFORMANCE_DATA); 
    return bFound; 

} // GetCounterName

